/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package algopher;

import java.util.*;

/** Scramble a cube with "length" moves.
 *
 * @author Michiel van der Blonk - blonkm@gmail.com
 */
public class Scrambler {
    /** the length (in nr of moves) for the scrambles */
    protected int length = 25;

    /** the actual moves in the scramble */
    private HashMap moves;
    private Move currMove = null;
    private Move prevMove = null;
    private enum states {first, newSlice, sameSlice}
    private states state = states.first;

    /** Create new Scrambler with given length
     *
     * @param length Number of moves in generated scrambles.
     */
    Scrambler(int length) {
        assert(length>=0);

        this.length = length;
    }

    /** Determine the state of the scramble at a given point.
     * This is done to prevent move sequences like "RL2R" and "LLRR"<br/>
     * 1) first move - any move is allowed<br/>
     * 2) move in new slice - previous is not allowed<br/>
     * 3) move in same slice as previous move (e.g. L,R) - same slice not allowed
     *
     * @param move current move in scramble
     * @param prevMove previous move in scramble, can be null on first move
     * @return 
     * null, at start <br/>
     * sameSlice, when both moves are in the same slice <br/>
     * newSlice, otherwise 
     */
    private states getState(Move move, Move prevMove) {
        states ret;

        if (move == null)
            ret = states.first;
        if (move.inSameSlice(prevMove))
            ret = states.sameSlice;
        else
            ret = states.newSlice;
        return ret;
    }

    /** find a next valid move in the scramble
     *
     * @return a valid next move, without repeating a move, or
     * a previous same-slice move
     */
    private Move nextMove() {
        String sep = ",";

        // initially all moves are valid
        HashMap<String, String[]> moveSet = new HashMap();
        moveSet.put("R", "R,R2,R'".split(sep));
        moveSet.put("L", "L,L2,L'".split(sep));
        moveSet.put("U", "U,U2,U'".split(sep));
        moveSet.put("D", "D,D2,D'".split(sep));
        moveSet.put("F", "F,F2,F'".split(sep));
        moveSet.put("B", "B,B2,B'".split(sep));

        // after first move, previous move is not allowed to repeat
        if (state!=states.first)
            moveSet.remove(currMove.face());
        // a move in the same slice is never allowed twice
        if (state==states.sameSlice) {
            moveSet.remove(prevMove.face());
            moveSet.remove(prevMove.opposite());
        }

        // get list of allowed face moves
        Object[] faces = moveSet.keySet().toArray();
        // pick a random face
        String face = (String)Picker.get(faces);
        // get allowed moves for this face
        String[] faceMoves = moveSet.get(face);
        // get a random move from allowed moves for this face
        Move move = new Move((String)Picker.get(faceMoves));
        return move;
     }

    /** get a random scramble for the Rubik's cube
     *
     * @return the scramble
     */
    public String getScramble() {
        String scramble = "";
        int n = this.length;

        assert (n>=0);

        while (n-- > 0) {
            currMove = nextMove();
            state = getState(currMove, prevMove);
            scramble += currMove + " ";
            // store this move
            prevMove = new Move(currMove.value);
        }

        // trim last added space
        return scramble.trim();
    }
}